"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const fs = __importStar(require("fs"));
const klaw_sync_1 = __importDefault(require("klaw-sync"));
const path = __importStar(require("path"));
const interfaces_1 = require("./interfaces");
const parse_1 = require("./parse");
function genPrefix(first, relativePath, ingoreDefaultMd) {
    const res = [];
    if (typeof first === 'string' && first.length > 0) {
        res.push(first);
    }
    res.push(...relativePath
        .split(path.sep)
        .filter(w => w.length > 0)
        .map(w => w.endsWith('.md') ? w.substr(0, w.length - 3) : w)
        .filter(w => w.length > 0));
    if (ingoreDefaultMd && res.length > 1 && res.includes('default')) {
        res.pop();
    }
    return res;
}
function cleanObject(obj) {
    const res = {};
    Object.keys(obj).forEach(key => {
        const { prefix, body, description, scope } = obj[key];
        res[key] = {
            prefix, body, description, scope
        };
    });
    return res;
}
function generator(config, options) {
    const cog = Object.assign({}, interfaces_1.DEFAULT_CONFIG, config);
    const sourcePaths = [];
    (typeof cog.sourceRoot === 'string' ? [cog.sourceRoot] : cog.sourceRoot)
        .map(p => path.resolve(process.cwd(), p))
        .filter(p => fs.existsSync(p))
        .forEach(rootPath => {
        klaw_sync_1.default(rootPath, {
            nodir: true,
            traverseAll: true,
            filter: (item) => item.path.endsWith('.md')
        })
            .map(item => item.path)
            .forEach(fullPath => {
            sourcePaths.push({ rootPath, fullPath });
        });
    });
    let res = {};
    sourcePaths.forEach(sourceItem => {
        const relativePath = path.relative(sourceItem.rootPath, sourceItem.fullPath);
        const md = fs.readFileSync(sourceItem.fullPath).toString('utf8');
        try {
            let item = parse_1.parse(md, sourceItem.fullPath, cog);
            const keys = genPrefix(cog.prefix, relativePath, cog.ingoreDefaultMd);
            item.prefix = keys.join(cog.separator);
            if (options && options.itemFinished) {
                item = options.itemFinished(item);
            }
            res[keys.join('_')] = item;
        }
        catch (err) {
            console.log(`>> File ${relativePath} parse error: ${chalk_1.default.red(err)}`);
        }
    });
    const successCount = Object.keys(res).length;
    const errorCount = sourcePaths.length - successCount;
    if (errorCount > 0) {
        console.log(`Find ${sourcePaths.length} markdowns, Success ${chalk_1.default.green(successCount + '')}, Error ${chalk_1.default.red(errorCount + '')}`);
    }
    else {
        console.log(`🌈  Find ${chalk_1.default.green(sourcePaths.length + '')} markdowns, ${chalk_1.default.green('All Success')}`);
    }
    res = cleanObject(res);
    if (options && options.finished) {
        res = options.finished(res);
    }
    const content = JSON.stringify(res, null, 2);
    const saveFile = path.resolve(process.cwd(), cog.outFile);
    if (fs.existsSync(saveFile)) {
        fs.unlinkSync(saveFile);
    }
    fs.writeFileSync(saveFile, content);
}
exports.generator = generator;
