#!/usr/bin/env node
"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const meow_1 = __importDefault(require("meow"));
const path = __importStar(require("path"));
const generator_1 = require("./generator");
const interfaces_1 = require("./interfaces");
const cli = meow_1.default({
    help: `
  Usage
    vscode-snippet-generator

  Example
    vscode-snippet-generator --outFile file.json

  Options
    -s, --sourceRoot  Source file root directory, can be set "src" "src1,src2", default: ${interfaces_1.DEFAULT_CONFIG.sourceRoot}

    -o, --outFile     Output file path, default: ${interfaces_1.DEFAULT_CONFIG.outFile}

    -p, --prefix      Unified prefix

    --separator       Multi-level directory separator, default: ${interfaces_1.DEFAULT_CONFIG.separator}

    --ingoreDefaultMd Whether to ignore "default.md" of the secondary directory, only keep directory name, default: ${interfaces_1.DEFAULT_CONFIG.ingoreDefaultMd}

    --i18n            Specify the language key name to generate, default: ${interfaces_1.DEFAULT_CONFIG.i18n}

    --config          A filepath of your config script
                      Example of a config script: https://github.com/cipchk/vscode-snippet-generator-tpl/blob/master/snippet-config.json
  `,
    flags: {
        sourceRoot: {
            type: 'string',
            default: interfaces_1.DEFAULT_CONFIG.sourceRoot,
            alias: 's'
        },
        outFile: {
            type: 'string',
            default: interfaces_1.DEFAULT_CONFIG.outFile,
            alias: 'o'
        },
        prefix: {
            type: 'string',
            alias: 'p'
        },
        separator: {
            type: 'string',
            default: interfaces_1.DEFAULT_CONFIG.separator
        },
        ingoreDefaultMd: {
            type: 'boolean',
            default: interfaces_1.DEFAULT_CONFIG.ingoreDefaultMd
        },
        i18n: {
            type: 'string',
            default: interfaces_1.DEFAULT_CONFIG.i18n
        },
        config: {
            type: 'string',
            default: 'snippet-config.json'
        }
    }
});
let configInFile;
try {
    const { config } = cli.flags;
    const configFile = path.resolve(process.cwd(), config);
    if (fs.existsSync(configFile)) {
        configInFile = require(configFile);
    }
}
catch (err) {
    console.error('Invalid config file', err);
    process.exit(1);
}
const options = Object.assign({}, interfaces_1.DEFAULT_CONFIG, configInFile, cli.flags);
const sourceRoot = options.sourceRoot.toString().trim();
if (sourceRoot.includes(',')) {
    options.sourceRoot = sourceRoot.split(',').map(p => p.trim());
}
generator_1.generator(options);
