"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const markdown_it_1 = __importDefault(require("markdown-it"));
const yamlFront = require('yaml-front-matter');
const path = __importStar(require("path"));
const interfaces_1 = require("./interfaces");
const it = new markdown_it_1.default({});
function isUndefined(meta, key) {
    return meta[key] != null;
}
function genDescriptionAndCode(tokens) {
    let description = '';
    const paragraphOpenTokenIndex = tokens.findIndex(w => w.type === 'paragraph_open');
    if (paragraphOpenTokenIndex !== -1 && tokens.length >= (paragraphOpenTokenIndex + 1)) {
        const paragraphToken = tokens[paragraphOpenTokenIndex + 1];
        description = paragraphToken.content.trim();
    }
    let body = '';
    const codeToken = tokens.find(w => w.tag === 'code');
    if (codeToken != null) {
        body = codeToken.content.trim();
    }
    return { description, body };
}
function fixPrefix(meta, fileName) {
    if (isUndefined(meta, 'prefix')) {
        return meta.prefix;
    }
    const name = fileName.split(path.sep).pop();
    return name.endsWith('.md') ? name.substr(0, name.length - 3) : name;
}
function fixDescription(tokens, meta, config) {
    if (!isUndefined(meta, 'description')) {
        meta.description = genDescriptionAndCode(tokens).description;
    }
    else if (typeof meta.description === 'object') {
        const source = meta.description;
        if (typeof config.i18nTpl === 'string' && config.i18nTpl.length > 0) {
            meta.description = config.i18nTpl.replace(/\{([^}]+)\}/g, (word, lang) => {
                return source[lang] || source[interfaces_1.DEFAULT_LANG];
            });
        }
        else {
            meta.description = source[config.i18n] || source[interfaces_1.DEFAULT_LANG];
        }
    }
}
function fixBody(tokens, meta, config) {
    const codeToken = tokens.find(w => w.tag === 'code');
    if (!codeToken) {
        throw new Error(`Must have a code body`);
    }
    const headingTokenPos = tokens
        .map((w, index) => w.type === 'heading_open' && (w.tag === 'h1' || w.tag === 'h2') ? index : -1)
        .filter(idx => idx !== -1)
        .map(start => {
        const end = tokens.slice(start).findIndex(w => w.tag === 'code');
        return { start, end: start + end + 1 };
    });
    if (headingTokenPos.length === 0) {
        meta.body = codeToken.content.trim();
        return;
    }
    for (const pos of headingTokenPos) {
        if (tokens[pos.start + 1].content.trim() !== config.i18n) {
            continue;
        }
        const { body, description } = genDescriptionAndCode(tokens.slice(pos.start, pos.end));
        meta.body = body;
        meta.description = description;
        break;
    }
}
function parse(md, fileName, config) {
    const meta = yamlFront.loadFront(md);
    const tokens = it.parse(meta.__content.toString(), {});
    delete meta.__content;
    fixBody(tokens, meta, config);
    if (typeof meta.description === 'object' || !isUndefined(meta, 'description')) {
        fixDescription(tokens, meta, config);
    }
    meta.prefix = fixPrefix(meta, fileName);
    return meta;
}
exports.parse = parse;
